@@ -6,11 +6,11 @@ package MarpaX::Languages::C::AST::Impl;
 # ABSTRACT: Implementation of Marpa's interface
 
 # use MarpaX::Languages::C::AST::Util qw/traceAndUnpack/;
-use Marpa::R2 2.078000;
+use Marpa::R2 2.081001;
 use Carp qw/croak/;
 use MarpaX::Languages::C::AST::Impl::Logger;
 
-our $VERSION = '0.35'; # VERSION
+our $VERSION = '0.37'; # VERSION
 
 our $MARPA_TRACE_FILE_HANDLE;
 our $MARPA_TRACE_BUFFER;
@@ -124,6 +124,16 @@ sub lexeme_read {
 }
 
 
+sub lexeme_alternative {
+  return $_[0]->{recce}->lexeme_alternative(@_[1..$#_]);
+}
+
+
+sub lexeme_complete {
+  return $_[0]->{recce}->lexeme_complete(@_[1..$#_]);
+}
+
+
 sub current_g1_location {
   return $_[0]->{recce}->current_g1_location(@_[1..$#_]);
 }
@@ -143,6 +153,31 @@ sub show_progress {
   return $_[0]->{recce}->show_progress(@_[1..$#_]);
 }
 
+
+sub start_symbol_id {
+  return $_[0]->{grammar}->start_symbol_id(@_[1..$#_]);
+}
+
+
+sub rule_ids {
+  return $_[0]->{grammar}->rule_ids(@_[1..$#_]);
+}
+
+
+sub symbol_name {
+  return $_[0]->{grammar}->symbol_name(@_[1..$#_]);
+}
+
+
+sub rule_name {
+  return $_[0]->{grammar}->rule_name(@_[1..$#_]);
+}
+
+
+sub rule_expand {
+  return $_[0]->{grammar}->rule_expand(@_[1..$#_]);
+}
+
 1;
 
 __END__
@@ -157,7 +192,7 @@ MarpaX::Languages::C::AST::Impl - Implementation of Marpa's interface
 
 =head1 VERSION
 
-version 0.35
+version 0.37
 
 =head1 SYNOPSIS
 
@@ -229,6 +264,14 @@ Returns Marpa's recognizer's substring corresponding to g1 span ($start, $length
 
 Returns Marpa's recognizer's lexeme_read for lexeme $lexeme, at start position $start, length $length and value $value.
 
+=head2 lexeme_alternative($self, $lexeme, $value)
+
+Returns Marpa's recognizer's lexeme_alternative for lexeme $lexeme, value $value.
+
+=head2 lexeme_complete($self, $start, $length)
+
+Returns Marpa's recognizer's lexeme_complete at start position $start, length $length.
+
 =head2 current_g1_location($self)
 
 Returns Marpa's recognizer's current_g1_location.
@@ -245,6 +288,26 @@ Returns Marpa's recognizer's terminals_expected.
 
 Returns Marpa's recognizer's show_progress.
 
+=head2 start_symbol_id($self)
+
+Returns Marpa's grammar's start_symbol_id.
+
+=head2 rule_ids($self)
+
+Returns Marpa's grammar's rule_ids.
+
+=head2 symbol_name($self)
+
+Returns Marpa's grammar's symbol_name.
+
+=head2 rule_name($self)
+
+Returns Marpa's grammar's rule_name.
+
+=head2 rule_expand($self)
+
+Returns Marpa's grammar's rule_expand.
+
 =head1 AUTHOR
 
 Jean-Damien Durand <jeandamiendurand@free.fr>
